use std::{ops::Deref, sync::Arc};

use aws_config::{BehaviorVersion, Region};
use aws_sdk_s3::config::Credentials;

use self::infra::aws_s3::s3_client;

#[derive(Clone)]
pub struct SharedApp(Arc<App>);

impl Deref for SharedApp {
    type Target = Arc<App>;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}

impl SharedApp {
    pub fn new(app: App) -> Self {
        Self(Arc::new(app))
    }
}

pub struct App {
    pub s3_client: aws_sdk_s3::Client,
    pub nats: infra::nats::Nats,
}

impl App {
    pub async fn new() -> anyhow::Result<Self> {
        Ok(Self {
            s3_client: s3_client().await?,
            nats: infra::nats::Nats::new().await?,
        })
    }
}

pub mod infra;
