use std::net::SocketAddr;

use tonic::transport::Server;

use crate::{
    app::SharedApp,
    services::release_manager::{
        extensions::ReleaseManagerExt, models::CommitArtifact, ReleaseManager,
    },
};

use self::gen::{greeter_server, HelloReply, HelloRequest};

mod gen {
    tonic::include_proto!("flux_releaser");
}

pub struct FluxReleaserGrpc {
    release_manager: ReleaseManager,
}

impl FluxReleaserGrpc {
    pub fn new(app: SharedApp) -> Self {
        Self {
            release_manager: app.release_manager(),
        }
    }
}

#[tonic::async_trait]
impl greeter_server::Greeter for FluxReleaserGrpc {
    async fn say_hello(
        &self,
        request: tonic::Request<HelloRequest>,
    ) -> std::result::Result<tonic::Response<HelloReply>, tonic::Status> {
        self.release_manager
            .commit_artifact(CommitArtifact {
                app: "some-app".into(),
                branch: "some-branch".into(),
            })
            .await
            .unwrap();

        Ok(tonic::Response::new(HelloReply {
            message: "something".into(),
        }))
    }
}

pub async fn tonic_serve(host: SocketAddr, app: SharedApp) -> anyhow::Result<()> {
    tracing::info!("grpc listening on: {}", host);
    Server::builder()
        .add_service(greeter_server::GreeterServer::new(FluxReleaserGrpc::new(
            app,
        )))
        .serve(host)
        .await?;

    Ok(())
}
